==> verser-0.0.3-py2.py3-none-any.whl: verser/verserLocal/requests_/my_cache.py <==
# from functools import lru_cache
import shutil
from pathlib import Path
from ..utils.utils_general import *

from typing import Union
from enum import Enum
import os
import requests
import pickle
import functools
from dataclasses import dataclass
from datetime import date, datetime

# -----------------------------------------------------------------------------
test_result_file_name = str(Path() / ".." / "requests_" / "test_reg_result")
arg_sep = "_argSEP_"
p = Path()
# pickle_folder = p.absolute() / 'test_pickles'
pickle_folder = p / 'test_pickles'

create_directory(pickle_folder)

print(pickle_folder)



# -----------------------------------------------------------------------------

class CacheDecider(Enum):
    day = date.today()
    hour = datetime.now().strftime("%H")
    nocache = datetime.now().strftime("%H:%M:%S")
    default = date.today()


def get_default_cache():
    obj = {
        "daily": CacheDecider.day,
        "hourly": CacheDecider.hour,
        'nocache': CacheDecider.nocache,
        'default': CacheDecider.day

    }
    return CacheDecider.day  # obj.get(options_.default_cache, CacheDecider.day)


def cache_represent(choice: CacheDecider):
    obj = {
        CacheDecider.day: "daily",
        CacheDecider.hour: "hourly",
        CacheDecider.nocache: 'nocache',
        CacheDecider.default: 'daily'
    }
    return obj[choice]


@dataclass
class MyCache:
    decider: Union[CacheDecider, None] = None  # post init

    # get_default_cache()  # get_default_cache()  # CacheDecider.hour
    # report_ = Report("stats_requests.txt")

    def __post_init__(self):
        # default_cache = SingletonOptions().get_valid_value("default_cache")
        # self.decider = get_enum_with_value(key=default_cache, enum_=CacheDecider, default_value=CacheDecider.day)
        ...

    def cache(self, func):

        @functools.wraps(func)
        def wrapper_decorator(*args, **kwargs):
            if self.check_cache(func, *args):
                value = self.load_cache(func, *args)

            else:
                value = func(*args, **kwargs)
                self.save_cache(func, *args, data=value)
            return value

        return wrapper_decorator

    def sort_safe(self, items):
        items = list(items)
        items = sorted(items)
        return items

    def safe_serialize(self, *items):
        safe_list = []
        for item in items:
            if isinstance(item, str):
                safe_list.append(item)

        safe_list = self.sort_safe(safe_list) + [str(self.decider)]
        parameters = arg_sep.join(safe_list)
        return parameters

    def get_hash_alt1(self, st):
        import hashlib, base64
        d = hashlib.md5(bytes(st, encoding="utf-8")).digest()
        d = base64.urlsafe_b64encode(d).decode('ascii')
        return d

    def get_func_hash(self, f, *args):
        parameters = self.safe_serialize(*args)
        f_hash = f"{f.__name__}__{parameters}"
        f_hash = self.get_hash_alt1(f_hash)
        return f_hash

    def check_cache(self, f, *args):
        """added below to provide an exception when we check for valid key """

        f_hash = self.get_func_hash(f, *args)
        f_hash_path = rf"{pickle_folder}\\{f_hash}"

        print(f_hash_path)
        return check_pickle(f_hash_path)

    def load_cache(self, f, *args):
        f_hash = self.get_func_hash(f, *args)
        f_hash_path = rf"{pickle_folder}\\{f_hash}"

        return load_pickle(f_hash_path)

    def save_cache(self, f, *args, data=None):
        f_hash = self.get_func_hash(f, *args)
        f_hash_path = f"{pickle_folder}\\{f_hash}"
        save_pickle(f_hash_path, data)


def check_pickle(file_name: str) -> bool:
    print("*"*100)
    print(file_name , "file_name")

    return os.path.isfile(file_name + ".pickle")


def load_pickle(file_name: str) -> Union[bool, requests.models.Response]:
    file_name = file_name + ".pickle"
    file_name_path = Path(file_name)

    if not file_name_path.is_file():
        print(file_name_path)
        return False
    with open(file_name_path, "rb") as infile:
        test_dict_reconstructed: requests.models.Response = pickle.load(infile)

        # exit()

    # f = bound(print, f"{file_name_path} pickle loaded...")
    # functools.partial(print, f"{file_name} pickle loaded...")
    # delay(1, f)
    msg = f"""
    
    ---------------------------------
    
    
    Recent cache was found for this request phrase.
    ---------------------------------
        It will be loaded instead of making a new redundant request 
        You may change cache options in options file.
    ---------------------------------
        cache loaded...({file_name_path})
"""

    print(msg)

    return test_dict_reconstructed


def delete_cache_folder():
    path = Path(pickle_folder)
    if not path.is_dir():
        return True
    try:
        # Path.rmdir(path)
        # os.remove(path)
        shutil.rmtree(path)
        return True
    except Exception as exc:
        print(exc)
    return False


def save_pickle(file_name, data):
    # print("saving", file_name)
    msg = f"""
        
    ---------------------------------
    
    
        Request was successfull...
        ---------------------------------
            Cache will be saved for this result. Next time program will be
            checking if this cache is new enough to load for the new request 
            You may change cache options in options file 
            ---------------------------------
            cache saved ...({file_name})
            
    """

    # file_name_path = pickle_folder / file_name + ".pickle"
    file_name_path =   file_name + ".pickle"
    try:
        save_pickle_helper(file_name_path, data)
    except:
        print(file_name_path)
        print("Cache folder may be deleted.\n setup_now() inorder to create...")
        # if callable("setup_now"):
        #     setup_now()
        #     save_pickle_helper(file_name_path, data)


def save_pickle_helper(file_name_path, data):
    with open(file_name_path, "wb") as outfile:
        pickle.dump(data, outfile)
    f = bound(print, f"{file_name_path} pickle saved...")
    delay(1, f)


def save_pickle_for_test(data):
    save_pickle(test_result_file_name, data)


def load_test_pickle():
    return load_pickle(test_result_file_name)


def lru_cache_patched(maxsize=128, typed=False):
    import weakref
    def decorator(func):
        @functools.wraps(func)
        def wrapped_func(self, *args, **kwargs):
            self_weak = weakref.ref(self)

            @functools.wraps(func)
            @functools.lru_cache(maxsize=maxsize, typed=typed)
            def cached_method(*args, **kwargs):
                return func(self_weak(), *args, **kwargs)

            setattr(self, func.__name__, cached_method)
            return cached_method(*args, **kwargs)

        return wrapped_func

    if callable(maxsize) and isinstance(typed, bool):
        func, maxsize = maxsize, 128
        return decorator(func)
    return decorator


__all__ = [
    'lru_cache_patched',
    'MyCache',
    'save_pickle_for_test',
    'load_test_pickle'
]

==> verser-0.0.5-py2.py3-none-any.whl: verser/verserLocal/requests_/my_cache.py <==
# from functools import lru_cache
import shutil
from pathlib import Path
from ..utils.utils_general import *

from typing import Union
from enum import Enum
import os
import requests
import pickle
import functools
from dataclasses import dataclass
from datetime import date, datetime

# -----------------------------------------------------------------------------
test_result_file_name = str(Path() / ".." / "requests_" / "test_reg_result")
arg_sep = "_argSEP_"
p = Path()
# pickle_folder = p.absolute() / 'test_pickles'
pickle_folder = p / 'test_pickles'

# create_directory(pickle_folder)
#
# print(pickle_folder)



# -----------------------------------------------------------------------------

class CacheDecider(Enum):
    day = date.today()
    hour = datetime.now().strftime("%H")
    nocache = datetime.now().strftime("%H:%M:%S")
    default = date.today()


def get_default_cache():
    obj = {
        "daily": CacheDecider.day,
        "hourly": CacheDecider.hour,
        'nocache': CacheDecider.nocache,
        'default': CacheDecider.day

    }
    return CacheDecider.day  # obj.get(options_.default_cache, CacheDecider.day)


def cache_represent(choice: CacheDecider):
    obj = {
        CacheDecider.day: "daily",
        CacheDecider.hour: "hourly",
        CacheDecider.nocache: 'nocache',
        CacheDecider.default: 'daily'
    }
    return obj[choice]


@dataclass
class MyCache:
    decider: Union[CacheDecider, None] = None  # post init

    # get_default_cache()  # get_default_cache()  # CacheDecider.hour
    # report_ = Report("stats_requests.txt")

    def __post_init__(self):
        # default_cache = SingletonOptions().get_valid_value("default_cache")
        # self.decider = get_enum_with_value(key=default_cache, enum_=CacheDecider, default_value=CacheDecider.day)
        ...

    def cache(self, func):

        @functools.wraps(func)
        def wrapper_decorator(*args, **kwargs):
            if self.check_cache(func, *args):
                value = self.load_cache(func, *args)

            else:
                value = func(*args, **kwargs)
                self.save_cache(func, *args, data=value)
            return value

        return wrapper_decorator

    def sort_safe(self, items):
        items = list(items)
        items = sorted(items)
        return items

    def safe_serialize(self, *items):
        safe_list = []
        for item in items:
            if isinstance(item, str):
                safe_list.append(item)

        safe_list = self.sort_safe(safe_list) + [str(self.decider)]
        parameters = arg_sep.join(safe_list)
        return parameters

    def get_hash_alt1(self, st):
        import hashlib, base64
        d = hashlib.md5(bytes(st, encoding="utf-8")).digest()
        d = base64.urlsafe_b64encode(d).decode('ascii')
        return d

    def get_func_hash(self, f, *args):
        parameters = self.safe_serialize(*args)
        f_hash = f"{f.__name__}__{parameters}"
        f_hash = self.get_hash_alt1(f_hash)
        return f_hash

    def check_cache(self, f, *args):
        """added below to provide an exception when we check for valid key """

        f_hash = self.get_func_hash(f, *args)
        f_hash_path = rf"{pickle_folder}\\{f_hash}"

        print(f_hash_path)
        return check_pickle(f_hash_path)

    def load_cache(self, f, *args):
        f_hash = self.get_func_hash(f, *args)
        f_hash_path = rf"{pickle_folder}\\{f_hash}"

        return load_pickle(f_hash_path)

    def save_cache(self, f, *args, data=None):
        f_hash = self.get_func_hash(f, *args)
        f_hash_path = f"{pickle_folder}\\{f_hash}"
        save_pickle(f_hash_path, data)


def check_pickle(file_name: str) -> bool:
    print("*"*100)
    print(file_name , "file_name")

    return os.path.isfile(file_name + ".pickle")


def load_pickle(file_name: str) -> Union[bool, requests.models.Response]:
    file_name = file_name + ".pickle"
    file_name_path = Path(file_name)

    if not file_name_path.is_file():
        print(file_name_path)
        return False
    with open(file_name_path, "rb") as infile:
        test_dict_reconstructed: requests.models.Response = pickle.load(infile)

        # exit()

    # f = bound(print, f"{file_name_path} pickle loaded...")
    # functools.partial(print, f"{file_name} pickle loaded...")
    # delay(1, f)
    msg = f"""
    
    ---------------------------------
    
    
    Recent cache was found for this request phrase.
    ---------------------------------
        It will be loaded instead of making a new redundant request 
        You may change cache options in options file.
    ---------------------------------
        cache loaded...({file_name_path})
"""

    print(msg)

    return test_dict_reconstructed


def delete_cache_folder():
    path = Path(pickle_folder)
    if not path.is_dir():
        return True
    try:
        # Path.rmdir(path)
        # os.remove(path)
        shutil.rmtree(path)
        return True
    except Exception as exc:
        print(exc)
    return False


def save_pickle(file_name, data):
    # print("saving", file_name)
    msg = f"""
        
    ---------------------------------
    
    
        Request was successfull...
        ---------------------------------
            Cache will be saved for this result. Next time program will be
            checking if this cache is new enough to load for the new request 
            You may change cache options in options file 
            ---------------------------------
            cache saved ...({file_name})
            
    """

    # file_name_path = pickle_folder / file_name + ".pickle"
    file_name_path =   file_name + ".pickle"
    try:
        save_pickle_helper(file_name_path, data)
    except:
        print(file_name_path)
        print("Cache folder may be deleted.\n setup_now() inorder to create...")
        # if callable("setup_now"):
        #     setup_now()
        #     save_pickle_helper(file_name_path, data)


def save_pickle_helper(file_name_path, data):
    with open(file_name_path, "wb") as outfile:
        pickle.dump(data, outfile)
    f = bound(print, f"{file_name_path} pickle saved...")
    delay(1, f)


def save_pickle_for_test(data):
    save_pickle(test_result_file_name, data)


def load_test_pickle():
    return load_pickle(test_result_file_name)


def lru_cache_patched(maxsize=128, typed=False):
    import weakref
    def decorator(func):
        @functools.wraps(func)
        def wrapped_func(self, *args, **kwargs):
            self_weak = weakref.ref(self)

            @functools.wraps(func)
            @functools.lru_cache(maxsize=maxsize, typed=typed)
            def cached_method(*args, **kwargs):
                return func(self_weak(), *args, **kwargs)

            setattr(self, func.__name__, cached_method)
            return cached_method(*args, **kwargs)

        return wrapped_func

    if callable(maxsize) and isinstance(typed, bool):
        func, maxsize = maxsize, 128
        return decorator(func)
    return decorator


__all__ = [
    'lru_cache_patched',
    'MyCache',
    'save_pickle_for_test',
    'load_test_pickle'
]
